///
/// VIA modoki
///
/// @author Sasaji
/// @date   2016.05.25
///

module MY_VIA (
	input wire clk48,		// 48MHz clock
	input wire clk,		// 1MHz clock
	input wire res_n,		// reset
	input wire cs_n,		// chip select
	input wire we_n,		// write enable
	input wire [3:0] rs,	// register select
	inout wire [7:0] da,	// data bus
	input wire [7:0]  pia,	// parallel input port A
	output wire [7:0] poa,	// parallel output port A
	input wire [7:0]  pib,	// parallel input port B
	output wire [7:0] pob,	// parallel output port B
	output wire irq		// interrupt
);

reg [7:0] da_reg;
reg [7:0] in_data[1:0];
reg [7:0] dir[1:0];	// i/o direction
reg [15:0] t1l;
reg [16:0] t1c;	// MSB is carry flag
reg t1c_active;
reg [7:0] t2l;
reg [16:0] t2c;	// MSB is carry flag
reg t2c_active;
reg [7:0] sr;
reg [7:0] acr;
reg [7:0] pcr;
reg [6:0] ifr;
reg [6:0] ier;
//wire irq;
`define CLK_NEG_CNTDWN 6'd12
reg [5:0] clk_neg_delay;
`define CLK_POS_WRITE 6'd10
`define CLK_POS_CLRCNT 6'd6
reg [5:0] clk_pos_delay;
//
`define t1_irq ifr[6]
`define t2_irq ifr[5]
`define sr_irq ifr[2]
//
assign irq = |(ifr[6:0] & ier[6:0]);
//assign irq_n = ~irq;
assign da = (~cs_n & we_n) ? da_reg : 8'bzzzzzzzz;
//
assign poa[7:0] = dir[0][7:0] & in_data[0][7:0];
assign pob[7:0] = dir[1][7:0] & in_data[1][7:0];
//
always @(posedge clk48, negedge res_n) begin
	if (~res_n) begin
		da_reg <= 8'h00;
		in_data[0] <= 8'h00;
		in_data[1] <= 8'h00;
		dir[0] <= 8'hff;
		dir[1] <= 8'hff;
		t1l <= 16'h0000;
		t1c <= 17'h00000;
		t1c_active <= 1'b0;
		t2l <= 8'h00;
		t2c <= 17'h00000;
		t2c_active <= 1'b0;
		sr <= 8'h00;
		acr <= 8'h00;
		pcr <= 8'h00;
		ifr <= 7'b0000000;
		ier <= 7'b0000000;
		clk_neg_delay <= 6'h0;
		clk_pos_delay <= 6'h0;
	end
	else begin
		if (clk) begin
			clk_pos_delay = clk_pos_delay + 6'h1;
			clk_neg_delay = 6'h0;
			// posedge clock
			if (~cs_n) begin
				// chipselect
				if (~we_n) begin
					if (clk_pos_delay == `CLK_POS_WRITE) begin
						// write to register
						case(rs)
						4'h0: begin
							// ORB
							in_data[1] = da;
						end
						4'h1: begin
							// ORA
							in_data[0] = da;
						end
						4'h2: begin
							// DDRB
							dir[1] = da;
						end
						4'h3: begin
							// DDRA
							dir[0] = da;
						end
						4'h4: begin
							// T1L L
							t1l[7:0] = da;
						end
						4'h5: begin
							// T1C H L 
							t1l[15:8] = da;
							t1c[16:8] = { 1'b0, da };
							`t1_irq = 1'b0;
							t1c_active = 1'b1;
							t1c[7:0] = t1l[7:0];
							t1c[16:0] = t1c[16:0] + 17'h1;
						end
						4'h6: begin
							// T1L L
							t1l[7:0] = da;
						end
						4'h7: begin
							// T1L H
							t1l[15:8] = da;
							`t1_irq = 1'b0;
						end
						4'h8: begin
							// T2L L
							t2l[7:0] = da;
						end
						4'h9: begin
							// T2C H
							`t2_irq = 1'b0;
							t2c_active = 1'b1;
							t2c[16:8] = { 1'b0, da };
							t2c[7:0] = t2l[7:0];
							t2c[16:0] = t2c[16:0] + 17'h1;
						end
						4'ha: begin
							// SR : Shift Register
							sr = da;
							`sr_irq = 1'b0;
						end
						4'hb: begin
							// ACR : Auxiliary Control Register
							acr = da;
						end
						4'hc: begin
							// PCR : Peripheral Control Register
							pcr = da;
						end
						4'hd: begin
							// IFR : Interrupt Flag Register
							ifr[6:0] = (ifr[6:0] & ~da[6:0]);
						end
						4'he: begin
							// IER : Interrupt Enable Register
							if (da[7]) ier[6:0] = da[6:0];
							else ier[6:0] = ~da[6:0];
						end
						4'hf: begin
							// ORA no effect
							in_data[0] = da;
						end
						endcase
					end
				end
				else begin
					// read from register
					case(rs)
					4'h0: begin
						// ORB
						da_reg = pib;
					end
					4'h1: begin
						// ORA
						da_reg = pia;
					end
					4'h2: begin
						// DDRB
						da_reg = dir[1];
					end
					4'h3: begin
						// DDRA
						da_reg = dir[0];
					end
					4'h4: begin
						// T1C L
						da_reg = t1c[7:0];
						`t1_irq = 1'b0;
					end
					4'h5: begin
						// T1C H 
						da_reg = t1c[15:8];
					end
					4'h6: begin
						// T1L L
						da_reg = t1l[7:0];
					end
					4'h7: begin
						// T1L H
						da_reg = t1l[15:8];
					end
					4'h8: begin
						// T2C L
						da_reg = t2c[7:0];
						`t2_irq = 1'b0;
					end
					4'h9: begin
						// T2C H
						da_reg = t2c[15:8];
					end
					4'ha: begin
						// SR : Shift Register
						da_reg = sr;
					end
					4'hb: begin
						// ACR : Auxiliary Control Register
						da_reg = acr;
					end
					4'hc: begin
						// PCR : Peripheral Control Register
						da_reg = pcr;
					end
					4'hd: begin
						// IFR : Interrupt Flag Register
						da_reg = { irq, ifr[6:0] };
					end
					4'he: begin
						// IER : Interrupt Enable Register
						da_reg = { 1'b0, ier[6:0] };
					end
					4'hf: begin
						// ORA no effect
						da_reg = pia;
					end
					endcase
				end
			end
			else begin
				if (clk_pos_delay == `CLK_POS_CLRCNT) begin
					// un chipselect
					if (t1c[16]) begin
						// set interrupt
						`t1_irq = t1c_active;
						if (acr[6] & t1c_active) begin
							// free-run mode
							t1c[16:0] = { 1'b0, t1l[15:0] };
							t1c[16:0] = t1c[16:0] + 17'h1;
						end
						else begin 
							t1c[16] = 1'b0;
							t1c_active = 1'b0;
						end
						// acr[7] is not supported.
					end
					if (t2c[16]) begin
						// set interrupt
						`t2_irq = t2c_active;
						t2c[16] = 1'b0;
						t2c_active = 1'b0;
					end
				end
			end
		end
		else begin
			clk_pos_delay <= 6'h0;
			clk_neg_delay <= clk_neg_delay + 6'h1;
			if (clk_neg_delay == `CLK_NEG_CNTDWN) begin
				// negedge clock
				t1c[16:0] = t1c[16:0] - 17'h1;
				t2c[16:0] = t2c[16:0] - 17'h1;
			end
		end
	end
end

endmodule
