`timescale 1ns/1ps
`define TB_CYCLE 20.83
`define E_CYCLE 1000
`define CLK2M_CYCLE 500

module L3_9PSG_tb();

reg clk48;
initial clk48 = 1'b0;
always #(`TB_CYCLE / 2) clk48 = ~clk48;

reg e;
initial e = 1'b0;
always #(`E_CYCLE / 2) e = ~e;

reg clk2m;
initial clk2m = 1'b0;
always #(`CLK2M_CYCLE / 2) clk2m = ~clk2m;

wire clk_main;
assign clk_main = uL3_9PSG.clk_main;

reg [31:0] cycles;
initial cycles = 32'h0;
always @(posedge clk_main) begin
	cycles = cycles + 32'h1;
end

reg sw_n;
reg [7:0] via_wdata;
reg [3:0] via_addr;
reg cs_n;
reg we_n;
reg res_n;

reg [15:0] phase;
reg [7:0] psg_adr;
reg [7:0] psg_dat;

reg [15:0] timer1;
reg [15:0] timer2;

wire irq_p;

initial
begin
	sw_n = 1'b1;
	via_wdata = 8'h00;
	via_addr = 4'd0;
	cs_n = 1'b1;
	we_n = 1'b1;
	res_n = 1'b1;

	phase = 16'd0;
	psg_adr = 8'h0;
	psg_dat = 8'h0;

	timer1 = 16'h0000;
	timer2 = 16'h0000;

	uL3_9PSG.uPOR.por_n_reg = 1'b0;
	uL3_9PSG.uPOR.por_count = 16'h0000;
end

`define PSG_SEL 3'b001

always @(posedge clk_main) begin

	if (phase > 0) begin
		phase = phase + 16'd1;
		case (phase)
		2: begin
			// latch address sel on PSG
			cs_n = 1'b0;
			via_addr = 4'd0;
			we_n = 1'b0;
			via_wdata = { 3'b000, `PSG_SEL, 2'b11 };
		end
		104: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		154: begin
			// latch via_address
			cs_n = 1'b0;
			via_addr = 4'd1;
			we_n = 1'b0;
			via_wdata = psg_adr;
		end
		256: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		306: begin
			// unselect PSG
			cs_n = 1'b0;
			via_addr = 4'd0;
			we_n = 1'b0;
			via_wdata = { 3'b000, `PSG_SEL, 2'b00 };
		end
		408: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		458: begin
			// write data
			cs_n = 1'b0;
			via_addr = 4'd1;
			we_n = 1'b0;
			via_wdata = psg_dat;
		end
		560: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		610: begin
			// write data sel on PSG
			cs_n = 1'b0;
			via_addr = 4'd0;
			we_n = 1'b0;
			via_wdata = { 3'b000, `PSG_SEL, 2'b01 };
		end
		712: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		752: begin
			// unselect PSG
			cs_n = 1'b0;
			via_addr = 4'd0;
			we_n = 1'b0;
			via_wdata = { 3'b000, `PSG_SEL, 2'b00 };
		end
		854: begin
			cs_n = 1'b1;
			we_n = 1'b1;
			via_wdata = 8'h00;
			phase = 16'd0;
		end
		// timer1
		1002: begin
			// ACR
			cs_n = 1'b0;
			via_addr = 4'd11;
			we_n = 1'b0;
			via_wdata = 8'b01000000;
		end
		1102: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		1152: begin
			// Enable Interrupt
			cs_n = 1'b0;
			via_addr = 4'd14;
			we_n = 1'b0;
			via_wdata = 8'b11100000;
		end
		1252: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		1302: begin
			// T1L-L
			cs_n = 1'b0;
			via_addr = 4'd4;
			we_n = 1'b0;
			via_wdata = timer1[7:0];
		end
		1402: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		1452: begin
			// T1L-H and Timer1 Start
			cs_n = 1'b0;
			via_addr = 4'd5;
			we_n = 1'b0;
			via_wdata = timer1[15:8];
		end
		1552: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		1602: begin
			// T2L-L
			cs_n = 1'b0;
			via_addr = 4'd8;
			we_n = 1'b0;
			via_wdata = timer2[7:0];
		end
		1702: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		1752: begin
			// Timer2 Start
			cs_n = 1'b0;
			via_addr = 4'd9;
			we_n = 1'b0;
			via_wdata = timer2[15:8];
		end
		1852: begin
			cs_n = 1'b1;
			we_n = 1'b1;
			phase = 16'd0;
		end

		// timer1 or 2 fired
		2002: begin
			// read T1C-H
			cs_n = 1'b0;
			via_addr = 4'd5;
			we_n = 1'b1;
		end
		2102: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		2152: begin
			// read T1C-L
			cs_n = 1'b0;
			via_addr = 4'd4;
			we_n = 1'b1;
		end
		2252: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		2302: begin
			// read T2C-H
			cs_n = 1'b0;
			via_addr = 4'd9;
			we_n = 1'b1;
		end
		2352: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		2402: begin
			// read T2C-L
			cs_n = 1'b0;
			via_addr = 4'd8;
			we_n = 1'b1;
		end
		2452: begin
			cs_n = 1'b1;
			we_n = 1'b1;
			phase = 16'd0;
		end

		// initialize
		10002: begin
			// DDR sel on port A
			cs_n = 1'b0;
			via_addr = 4'd3;
			we_n = 1'b0;
			via_wdata = 8'b11111111;
		end
		10102: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end
		10152: begin
			// DDR sel on port B
			cs_n = 1'b0;
			via_addr = 4'd2;
			we_n = 1'b0;
			via_wdata = 8'b11111111;
		end
		10252: begin
			cs_n = 1'b1;
			we_n = 1'b1;
		end

		endcase
	end
	else if (irq_p) begin
		phase = 16'd2001;
	end

	case (cycles)
	81000: begin
		phase = 16'd10001;
	end

	82000: begin
		// PSG volume
		psg_adr = 8'd7;
		psg_dat = 8'hd8;
		phase = 16'h1;
	end

	83000: begin
		// PSG envelop freq L
		psg_adr = 8'd11;
		psg_dat = 8'h4e;
		phase = 16'h1;
	end
	84000: begin
		// PSG envelop freq H
		psg_adr = 8'd12;
		psg_dat = 8'h00;
		phase = 16'h1;
	end
	85000: begin
		// PSG envelop type
		psg_adr = 8'd13;
		psg_dat = 8'h0e;
		phase = 16'h1;
	end

	86000: begin
		// PSG noise freq
		psg_adr = 8'd6;
		psg_dat = 8'h10;
		phase = 16'h1;
	end

	87000: begin
		// PSG chA freq
		psg_adr = 8'd0;
		psg_dat = 8'hef;
		phase = 16'h1;
	end
	88000: begin
		psg_adr = 8'd1;
		psg_dat = 8'h0;
		phase = 16'h1;
	end
	89000: begin
		// PSG chA volume
		psg_adr = 8'd8;
		psg_dat = 8'h1f;
		phase = 16'h1;
	end

	90000: begin
		// PSG chB freq
		psg_adr = 8'd2;
		psg_dat = 8'hd5;
		phase = 16'h1;
	end
	91000: begin
		psg_adr = 8'd3;
		psg_dat = 8'h0;
		phase = 16'h1;
	end
	92000: begin
		// PSG chB volume
		psg_adr = 8'd9;
		psg_dat = 8'hc;
		phase = 16'h1;
	end

	93000: begin
		// PSG chC freq
		psg_adr = 8'd4;
		psg_dat = 8'hbe;
		phase = 16'h1;
	end
	94000: begin
		psg_adr = 8'd5;
		psg_dat = 8'h0;
		phase = 16'h1;
	end
	95000: begin
		// PSG chC volume
		psg_adr = 8'd10;
		psg_dat = 8'hb;
		phase = 16'h1;
	end

	// timer1  test
	100000: begin
		timer1 = 16'd1024;
		timer2 = 16'd3000;
		phase = 16'd1001;
	end

	endcase
end

wire [2:0] led;
wire [7:0] via_data;
L3_9PSG #(1) uL3_9PSG
(
	.clk48		(clk48),
	.sw_n		(sw_n),
	.data		(via_data),
	.addr		(via_addr),
	.cs_n		(cs_n),
	.we_n		(we_n),
	.e		(e),
	.clk2m		(clk2m),
	.res_p		(~res_n),
	.led		(led),
	.snd_out_0	(snd_out_0),
	.snd_out_1	(snd_out_1),
	.snd_out_2	(snd_out_2),
	.irq_p		(irq_p)
);
assign via_data = ~we_n ? via_wdata : 8'bzzzzzzzz;

endmodule

